"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createApi = void 0;
const serializefunc_1 = require("./serializefunc");
const utils_1 = require("./utils");
const config_1 = require("./config");
const createApi = (models, bundle) => {
    return `/* tslint:disable */
/* eslint-disable */
    
import * as models from './models';
import * as runtime from './runtime';
import {${serializefunc_1.THROW_NULLABLE_ERROR_METHOD_NAME}} from './models'; 
    
${bundle.calls.map(createRequestResponse).join('\n')}
     
export class ${(0, utils_1.pascalCase)(bundle.name)}API extends runtime.BaseAPI {
    ${bundle.calls.map((call) => createCall(models, call)).join('\n    ')}
}
`;
};
exports.createApi = createApi;
const createCall = (models, call) => {
    return `
    async ${call.nickname}Raw(request: ${(0, utils_1.pascalCase)(call.nickname)}Request${call.body ? ', body: ' + getType(call.body) : ''}): Promise<runtime.ApiResponse<${(0, utils_1.pascalCase)(call.nickname)}Response>> {
        const queryParameters: runtime.HTTPQuery = {};
        ${createQueryParams(call)}
        const headerParameters: runtime.HTTPHeaders = {};
        headerParameters['Content-Type'] = 'application/json';

        const response = await this.request({
            path: ${createPath(call)},
            method: '${call.method}',
            headers: headerParameters,
            query: queryParameters,
            ${call.body ? 'body: ' + (0, utils_1.property)(models, call.body, true, 'body', 'models.') + ',' : ''}
        });

        return new runtime.${call.returnType ? 'JSON' : 'Void'}ApiResponse${call.returnType ? '<any>' : ''}(response${call.returnType ? ', (value) => ' + (0, utils_1.property)(models, call.returnType, false, 'value', 'models.') : ''});
    }
    async ${call.nickname}(request: ${(0, utils_1.pascalCase)(call.nickname)}Request${call.body ? ', body: ' + getType(call.body) : ''}): Promise<${(0, utils_1.pascalCase)(call.nickname)}Response> {
        const response = await this.${call.nickname}Raw(request${call.body ? ', body' : ''});
        return await response.value();
    }
    `;
};
const createPath = (call) => {
    if (!call.params.some((param) => param.in === 'path')) {
        return `'${call.path}'`;
    }
    return (`'${call.path}'.` +
        call.params
            .filter((param) => param.in === 'path')
            .map((param) => `replace("{${param.name}}", encodeURIComponent(request['${param.interfaceName}']))`)
            .join('.'));
};
const createQueryParams = (call) => {
    const requiredStr = (param) => (param.required ? '' : `if (request.${param.interfaceName} !== undefined)`);
    return call.params
        .filter((param) => param.in === 'query')
        .map((param) => requiredStr(param) + `queryParameters['${param.name}'] = request['${param.interfaceName}'];`)
        .join('\n        ');
};
const createRequestResponse = (call) => {
    return `
export interface ${(0, utils_1.pascalCase)(call.nickname)}Request {
    ${call.params.map(createParam).join('\n    ')}
}

export type ${(0, utils_1.pascalCase)(call.nickname)}Response = ${getType(call.returnType)};
`;
};
const createParam = (params) => `${params.interfaceName}${params.required ? ':' : '?:'} ${getType(params.type)};`;
const getType = (s) => {
    if (!s) {
        return 'void';
    }
    if (config_1.config.omitProdDeserialize && s === config_1.config.dateType) {
        return 'ApiDate';
    }
    return utils_1.primitives.some((o) => s.startsWith(o)) ? s : 'models.' + s;
};
//# sourceMappingURL=templateApi.js.map